import customtkinter as ctk
import tkinter as tk
from tkinter import filedialog, messagebox
import pandas as pd
import joblib
import os
from tkinter import ttk
from sklearn.base import BaseEstimator, TransformerMixin
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from reportlab.lib import colors
from reportlab.lib.pagesizes import letter
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Image, Table, TableStyle
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import inch
import matplotlib.pyplot as plt
from reportlab.pdfgen import canvas
from reportlab.lib.utils import ImageReader
import io
# Set appearance and current directory & model path
ctk.set_appearance_mode("System")
current_dir = os.path.dirname(os.path.abspath(__file__))
model_path = os.path.join(current_dir, "random_forest_pipeline.pkl")

# Window dimensions
appwidth, appheight = 1000, 600
# Colors
BACKGROUND_COLOR = "#2E2E2E"
NAV_BAR_COLOR = "#1C1C1C"
BUTTON_COLOR = "#DB3E39"
BUTTON_HOVER_COLOR = "#821D1A"
TEXT_COLOR = "white"
ROW_HIGHLIGHT_COLOR = "#505050"  # Highlight on hover

# ------------------ Helper Function ------------------
def center_window(window, width, height):
    screen_width = window.winfo_screenwidth()
    screen_height = window.winfo_screenheight()
    x = (screen_width // 2) - (width // 2)
    y = (screen_height // 2) - (height // 2)
    window.geometry(f"{width}x{height}+{x}+{y}")

# ------------------ Navigation Bar ------------------
class Dashboard_Frame(ctk.CTkFrame):
    def __init__(self, master, app, **kwargs):
        super().__init__(master, **kwargs)
        self.app = app

        # Set background color to make it seamless with the rest of the app
        self.configure(fg_color="transparent")  # or replace "transparent" with your desired color

        # Centered buttons in one row (as in your original design)
        self.dashButton = ctk.CTkButton(
            self,
            text="Dashboard",
            fg_color=("#DB3E39", "#821D1A"),
            hover_color="black",
            corner_radius=20,
            width=100,
            height=30,
            command=lambda: self.app.show_page("dashboard")
        )
        self.dashButton.grid(row=0, column=0, padx=20)

        self.viewButton = ctk.CTkButton(
            self,
            text="View",
            fg_color=("#DB3E39", "#821D1A"),
            hover_color="black",
            corner_radius=20,
            width=100,
            height=30,
            command=lambda: self.app.show_page("view")
        )
        self.viewButton.grid(row=0, column=1, padx=20)

        self.uploadButton = ctk.CTkButton(
            self,
            text="Upload",
            fg_color=("#DB3E39", "#821D1A"),
            hover_color="black",
            corner_radius=20,
            width=100,
            height=30,
            command=lambda: self.app.show_page("upload")
        )
        self.uploadButton.grid(row=0, column=2, padx=20)

        self.helpButton = ctk.CTkButton(
            self,
            text="Help",
            fg_color=("#DB3E39", "#821D1A"),
            hover_color="black",
            corner_radius=20,
            width=100,
            height=30,
            command=lambda: self.app.show_page("help")
        )

        self.helpButton.grid(row=0, column=3, padx=20)

# ------------------ Upload Page ------------------

class UploadPage(ctk.CTkFrame):
    def __init__(self, parent, controller):
        super().__init__(parent)
        self.controller = controller
        # Set background color to make it seamless with the rest of the app
        self.configure(fg_color="transparent")  # or replace "transparent" with your desired color

        self.generateResultsButton = ctk.CTkButton(
            self,
            text="Upload CSV FILE",
            fg_color=("#DB3E39", "#821D1A"),
            hover_color="black",
            command=self.controller.upload_csv_file
        )
        self.generateResultsButton.place(relx=0.5, rely=0.45, anchor="center")
        
        self.exportButton = ctk.CTkButton(
            self,
            text="Download CSV FILE",
            fg_color=("#DB3E39", "#821D1A"),
            hover_color="black",
            command=self.controller.export_csv_file,
            state=ctk.DISABLED
        )
        self.exportButton.place(relx=0.5, rely=0.55, anchor="center")
        
        self.generateReportButton = ctk.CTkButton(
            self,
            text="Generate Report",
            fg_color=("#DB3E39", "#821D1A"),
            hover_color="black",
            command=self.generate_report,
            state=ctk.DISABLED
        )
        self.generateReportButton.place(relx=0.5, rely=0.65, anchor="center")
        
        self.status_label = ctk.CTkLabel(self, text="No file uploaded yet.")
        self.status_label.place(relx=0.5, rely=0.35, anchor="center")
    
    def set_status(self, text):
        self.status_label.configure(text=text)
    
    def set_export_state(self, state):
        self.exportButton.configure(state=state)
        self.generateReportButton.configure(state=state)  # Enable/disable Generate Report button
    
    def generate_report(self):
        """Generate a PDF report with charts, total patient counts, and Need Referred patient data."""
        if self.controller.new_data is None:
            messagebox.showwarning("No Data", "No data available to generate a report.")
            return
        
        # Ask user for the file path to save the report
        file_path = filedialog.asksaveasfilename(
            defaultextension=".pdf",
            filetypes=[("PDF Files", "*.pdf")],
            title="Save Report As"
        )
        if not file_path:
            return  # User canceled the save dialog
        
        try:
            # Create a PDF document
            doc = SimpleDocTemplate(file_path, pagesize=letter, leftMargin=20, rightMargin=20)
            styles = getSampleStyleSheet()
            story = []

            # Add title
            title = Paragraph("Medical Dashboard Report", styles["Title"])
            story.append(title)
            story.append(Spacer(1, 12))

            # Add total patient counts
            result_counts = self.controller.new_data["Result"].value_counts()
            total_patients = len(self.controller.new_data)
            need_referred = result_counts.get("Need Referred", 0)
            referred = result_counts.get("Referred", 0)
            not_need_referred = result_counts.get("Do Not Need Referred", 0)

            counts_text = f"""
            <b>Total Patients:</b> {total_patients}<br/>
            <b>Need Referred:</b> {need_referred}<br/>
            <b>Referred:</b> {referred}<br/>
            <b>Do Not Need Referred:</b> {not_need_referred}
            """
            counts_paragraph = Paragraph(counts_text, styles["BodyText"])
            story.append(counts_paragraph)
            story.append(Spacer(1, 24))

            # Add charts
            story.append(Paragraph("<b>Charts</b>", styles["Heading2"]))
            story.append(Spacer(1, 12))

            # Temporary file paths
            pie_chart_path = "pie_chart.png"
            bar_chart_zero_path = "bar_chart_zero.png"
            bar_chart_avg_path = "bar_chart_avg.png"

            # ------------------ Pie Chart ------------------
            if hasattr(self.controller.pages["dashboard"], "pie_frame"):
                fig1, ax1 = plt.subplots(figsize=(4, 4), dpi=80, facecolor="black")
                result_counts = self.controller.new_data["Result"].value_counts()
                labels = result_counts.index.tolist()
                sizes = result_counts.values
                ax1.pie(
                    sizes, labels=labels, autopct="%1.0f%%",
                    colors=["#3498db", "#e74c3c", "#FFC107"], startangle=90,
                    wedgeprops={"edgecolor": "white"}, textprops={'fontsize': 10, 'color': 'white'}
                )
                ax1.set_title("Result Distribution", fontsize=12, fontweight="bold", color="white")
                fig1.savefig(pie_chart_path, bbox_inches="tight", facecolor="black")
                plt.close(fig1)

                # Add pie chart to PDF
                pie_chart_image = Image(pie_chart_path, width=4 * inch, height=4 * inch)
                story.append(pie_chart_image)
                story.append(Spacer(1, 12))

            # ------------------ Zero Values Bar Chart ------------------
            if hasattr(self.controller.pages["dashboard"], "bar_frame_zero"):
                target_columns_zero = [
                    "end_tidal_co2", "feed_vol", "oxygen_flow_rate", "fio2",
                    "peep", "tidal_vol", "pip", "sip", "insp_time"
                ]
                filtered_data_zero = self.controller.new_data[target_columns_zero]
                zero_counts = (filtered_data_zero == 0.0).sum()
                columns_zero = zero_counts.index.tolist()
                counts_zero = zero_counts.values

                fig2, ax2 = plt.subplots(figsize=(6, 5), dpi=80)
                bars_zero = ax2.bar(columns_zero, counts_zero, color="#26C6DA")
                ax2.set_title("Zero Values in Key Columns", fontsize=12, color=TEXT_COLOR)
                ax2.set_ylabel("Count of Zero Values", fontsize=10, color=TEXT_COLOR)
                ax2.set_xlabel("Columns", fontsize=10, color=TEXT_COLOR)
                ax2.tick_params(axis="x", rotation=90, colors=TEXT_COLOR)
                ax2.tick_params(axis="y", colors=TEXT_COLOR)
                ax2.set_facecolor(NAV_BAR_COLOR)
                fig2.patch.set_facecolor(NAV_BAR_COLOR)
                fig2.savefig(bar_chart_zero_path, bbox_inches="tight", facecolor=NAV_BAR_COLOR)
                plt.close(fig2)

                # Add bar chart to PDF
                bar_chart_zero_image = Image(bar_chart_zero_path, width=6 * inch, height=4 * inch)
                story.append(bar_chart_zero_image)
                story.append(Spacer(1, 12))

            # ------------------ Average Values Bar Chart ------------------
            if hasattr(self.controller.pages["dashboard"], "bar_frame_avg"):
                target_columns_avg = [
                    "end_tidal_co2", "feed_vol", "oxygen_flow_rate", "fio2",
                    "peep", "tidal_vol", "pip", "sip", "insp_time"
                ]
                filtered_data_avg = self.controller.new_data[target_columns_avg]
                avg_values = filtered_data_avg.mean()
                columns_avg = avg_values.index.tolist()
                values_avg = avg_values.values

                fig3, ax3 = plt.subplots(figsize=(8, 5), dpi=80)
                bars_avg = ax3.bar(columns_avg, values_avg, color="#FFC107")
                ax3.set_title("Average Values in Key Columns", fontsize=12, color=TEXT_COLOR)
                ax3.set_ylabel("Average Value", fontsize=10, color=TEXT_COLOR)
                ax3.set_xlabel("Columns", fontsize=10, color=TEXT_COLOR)
                ax3.tick_params(axis="x", rotation=90, colors=TEXT_COLOR)
                ax3.tick_params(axis="y", colors=TEXT_COLOR)
                ax3.set_facecolor(NAV_BAR_COLOR)
                fig3.patch.set_facecolor(NAV_BAR_COLOR)
                fig3.savefig(bar_chart_avg_path, bbox_inches="tight", facecolor=NAV_BAR_COLOR)
                plt.close(fig3)

                # Add bar chart to PDF (with fixed width)
                bar_chart_avg_image = Image(bar_chart_avg_path, width=6 * inch, height=4 * inch)
                story.append(bar_chart_avg_image)
                story.append(Spacer(1, 12))

            # ------------------ Need Referred Patient Data ------------------
            need_referred_data = self.controller.new_data[self.controller.new_data["Result"] == "Need Referred"]
            if not need_referred_data.empty:
                # Select only the specified columns
                selected_columns = ["encounterId", "Result"]
                need_referred_data = need_referred_data[selected_columns]

                # Format floating-point numbers to show only 2 decimal places
                for col in need_referred_data.columns:
                    if need_referred_data[col].dtype == "float64":
                        need_referred_data[col] = need_referred_data[col].apply(lambda x: f"{x:.2f}")

                story.append(Paragraph("<b>Need Referred Patients Data</b>", styles["Heading2"]))
                story.append(Spacer(1, 12))

                # Convert data to a list of lists for the table
                data = [need_referred_data.columns.tolist()] + need_referred_data.values.tolist()

                # Use a fixed column width for all columns
                col_widths = [0.8 * inch] * len(need_referred_data.columns)

                # Create a table with fixed column widths and smaller font size
                table = Table(data, colWidths=col_widths)
                table.setStyle(TableStyle([
                    ("BACKGROUND", (0, 0), (-1, 0), colors.grey),
                    ("TEXTCOLOR", (0, 0), (-1, 0), colors.whitesmoke),
                    ("ALIGN", (0, 0), (-1, -1), "CENTER"),
                    ("FONTNAME", (0, 0), (-1, 0), "Helvetica-Bold"),
                    ("FONTSIZE", (0, 0), (-1, -1), 6),
                    ("BOTTOMPADDING", (0, 0), (-1, 0), 6),
                    ("BACKGROUND", (0, 1), (-1, -1), colors.beige),
                    ("GRID", (0, 0), (-1, -1), 0.5, colors.black),
                    ("WORDWRAP", (0, 0), (-1, -1), True),
                ]))
                story.append(table)

            # Build the PDF
            doc.build(story)
            
            # Clean up temporary image files
            for img_path in [pie_chart_path, bar_chart_zero_path, bar_chart_avg_path]:
                if os.path.exists(img_path):
                    os.remove(img_path)
                    
            messagebox.showinfo("Report Generated", f"Report saved successfully at:\n{file_path}")
        except Exception as e:
            messagebox.showerror("Error", f"Failed to generate report: {e}")






class DashboardPage(ctk.CTkFrame):
    def __init__(self, parent, controller):
        super().__init__(parent)
        self.controller = controller
        self.configure(fg_color=BACKGROUND_COLOR)  # Use the app's background color

        # Create a scrollable container
        self.scrollable_frame = ctk.CTkScrollableFrame(self, fg_color=BACKGROUND_COLOR)
        self.scrollable_frame.pack(fill="both", expand=True)

        # Title for the dashboard
        self.title_label = ctk.CTkLabel(
            self.scrollable_frame,
            text="Dashboard",
            font=("Arial", 24, "bold"),
            text_color=TEXT_COLOR
        )
        self.title_label.pack(pady=(0, 20))

        # ------------------ Summary Card ------------------
        # Create a frame for the summary card
        self.summary_card = ctk.CTkFrame(self.scrollable_frame, fg_color=NAV_BAR_COLOR, corner_radius=10)
        self.summary_card.pack(fill="x", padx=10, pady=(0, 20))

        # Add labels for the counts
        self.total_patients_label = ctk.CTkLabel(
            self.summary_card,
            text="Total Patients: 0",
            font=("Arial", 16, "bold"),
            text_color=TEXT_COLOR
        )
        self.total_patients_label.pack(side="left", padx=20, pady=10)

        self.need_referred_label = ctk.CTkLabel(
            self.summary_card,
            text="Need Referred: 0",
            font=("Arial", 16, "bold"),
            text_color="#FFC107"  # Blue color for "Need Referred"
        )
        self.need_referred_label.pack(side="left", padx=20, pady=10)

        self.referred_label = ctk.CTkLabel(
            self.summary_card,
            text="Referred: 0",
            font=("Arial", 16, "bold"),
            text_color="#e74c3c"  # Red color for "Referred"
        )
        self.referred_label.pack(side="left", padx=20, pady=10)

        self.not_need_referred_label = ctk.CTkLabel(
            self.summary_card,
            text="Do Not Need Referred: 0",
            font=("Arial", 16, "bold"),
            text_color="#3498db"  # Yellow color for "Do Not Need Referred"
        )
        self.not_need_referred_label.pack(side="left", padx=20, pady=10)

        # ------------------ Chart Grid ------------------
        # Grid layout for charts
        self.chart_grid = ctk.CTkFrame(self.scrollable_frame, fg_color=BACKGROUND_COLOR)
        self.chart_grid.pack(fill="both", expand=True)

        # Pie Chart Frame
        self.pie_frame = ctk.CTkFrame(self.chart_grid, fg_color="black", corner_radius=10, height=400)  # Fixed height
        self.pie_frame.grid(row=0, column=0, padx=10, pady=10, sticky="nsew")

        # Bar Chart Frame (for zero values, taller height)
        self.bar_frame_zero = ctk.CTkFrame(self.chart_grid, fg_color="black", corner_radius=10, height=500)  # Fixed height
        self.bar_frame_zero.grid(row=0, column=1, padx=10, pady=10, sticky="nsew")

        # Bar Chart Frame (for average values, taller height)
        self.bar_frame_avg = ctk.CTkFrame(self.chart_grid, fg_color="black", corner_radius=10, height=500)  # Fixed height
        self.bar_frame_avg.grid(row=1, column=0, columnspan=2, padx=10, pady=10, sticky="nsew")

        # Configure grid weights for responsiveness
        self.chart_grid.rowconfigure(0, weight=1)
        self.chart_grid.rowconfigure(1, weight=1)
        self.chart_grid.columnconfigure(0, weight=1)
        self.chart_grid.columnconfigure(1, weight=1)

        # Bind window resize event
        self.bind("<Configure>", self._on_window_resize)
        self.resize_timer = None

    def _on_window_resize(self, event):
        """Handle window resize events to avoid laggy chart redrawing."""
        if self.resize_timer:
            self.after_cancel(self.resize_timer)  # Cancel the previous timer
        # Redraw charts after 200ms of no resizing
        self.resize_timer = self.after(200, self._redraw_charts)

    def _redraw_charts(self):
        """Redraw charts after window resizing is complete."""
        if hasattr(self, "current_data"):
            self.update_charts(self.current_data)

    def update_charts(self, data):
        """Update the charts with new data."""
        self.current_data = data  # Cache the data for redrawing

        # Update summary card with counts
        if data is not None and "Result" in data.columns:
            result_counts = data["Result"].value_counts()
            total_patients = len(data)
            need_referred = result_counts.get("Need Referred", 0)
            referred = result_counts.get("Referred", 0)
            not_need_referred = result_counts.get("Do Not Need Referred", 0)

            # Update the labels in the summary card
            self.total_patients_label.configure(text=f"Total Patients: {total_patients}")
            self.need_referred_label.configure(text=f"Need Referred: {need_referred}")
            self.referred_label.configure(text=f"Referred: {referred}")
            self.not_need_referred_label.configure(text=f"Do Not Need Referred: {not_need_referred}")

        # Clear previous charts
        for widget in self.pie_frame.winfo_children():
            widget.destroy()
        for widget in self.bar_frame_zero.winfo_children():
            widget.destroy()
        for widget in self.bar_frame_avg.winfo_children():
            widget.destroy()

        # Update Pie Chart (with new styling and larger size)
        if data is not None and "Result" in data.columns:
            result_counts = data["Result"].value_counts()
            labels = result_counts.index.tolist()
            sizes = result_counts.values

            # Pie Chart with new styling and larger size
            fig1, ax1 = plt.subplots(figsize=(4, 4), dpi=80, facecolor="black")  # Smaller size
            wedges, texts, autotexts = ax1.pie(
                sizes, labels=labels, autopct="%1.0f%%",
                colors=["#3498db", "#e74c3c", "#FFC107"], startangle=90,
                wedgeprops={"edgecolor": "white"}, textprops={'fontsize': 10, 'color': 'white'}  # Smaller font size
            )
            ax1.set_title("Result Distribution", fontsize=12, fontweight="bold", color="white")  # Smaller title font size
            fig1.patch.set_facecolor("black")

            # Add interactivity to pie chart (tooltips and clickable legend)
            def on_pie_click(event):
                if event.inaxes == ax1:
                    for i, wedge in enumerate(wedges):
                        if wedge.contains(event)[0]:
                            texts[i].set_visible(not texts[i].get_visible())  # Toggle label visibility
                            autotexts[i].set_visible(not autotexts[i].get_visible())  # Toggle percentage visibility
                            fig1.canvas.draw()

            fig1.canvas.mpl_connect("button_press_event", on_pie_click)

            canvas1 = FigureCanvasTkAgg(fig1, self.pie_frame)
            canvas1.draw()
            canvas1.get_tk_widget().pack(expand=True, fill="both", padx=10, pady=10)

            # Connect hover event for pie chart
            def on_pie_hover(event):
                if event.inaxes == ax1:
                    for wedge in wedges:
                        if wedge.contains(event)[0]:
                            wedge.set_alpha(0.7)  # Highlight the wedge
                        else:
                            wedge.set_alpha(1.0)  # Reset other wedges
                    canvas1.draw()

            fig1.canvas.mpl_connect("motion_notify_event", on_pie_hover)

        # New Bar Chart for Zero Values (specific columns only)
        if data is not None:
            # List of columns to include in the bar chart
            target_columns_zero = [
                "end_tidal_co2", "feed_vol", "oxygen_flow_rate", "fio2",
                "peep", "tidal_vol", "pip", "sip", "insp_time"
            ]

            # Filter the dataset to include only the target columns
            filtered_data_zero = data[target_columns_zero]

            # Count `0.0` values for each target column (treat `0.0` as missing/null)
            zero_counts = (filtered_data_zero == 0.0).sum()
            columns_zero = zero_counts.index.tolist()
            counts_zero = zero_counts.values

            # Bar Chart for Zero Values
            fig2, ax2 = plt.subplots(figsize=(6, 5), dpi=80)  # Taller chart
            bars_zero = ax2.bar(columns_zero, counts_zero, color="#26C6DA")
            ax2.set_title("Zero Values in Key Columns", fontsize=12, color=TEXT_COLOR)
            ax2.set_ylabel("Count of Zero Values", fontsize=10, color=TEXT_COLOR)
            ax2.set_xlabel("Columns", fontsize=10, color=TEXT_COLOR)
            ax2.tick_params(axis="x", rotation=90, colors=TEXT_COLOR)  # Rotate x-axis labels
            ax2.tick_params(axis="y", colors=TEXT_COLOR)
            ax2.set_facecolor(NAV_BAR_COLOR)
            fig2.patch.set_facecolor(NAV_BAR_COLOR)

            # Add interactivity to bar chart (tooltips and zoom/pan)
            for bar in bars_zero:
                height = bar.get_height()
                ax2.text(
                    bar.get_x() + bar.get_width() / 2, height,
                    f"{int(height)}", ha="center", va="bottom", color=TEXT_COLOR
                )
                bar.set_picker(True)  # Enable picking for hover events

            canvas2 = FigureCanvasTkAgg(fig2, self.bar_frame_zero)
            canvas2.draw()
            canvas2.get_tk_widget().pack(expand=True, fill="both", padx=10, pady=10)

            # Connect hover event for bar chart
            def on_bar_hover(event):
                if event.inaxes == ax2:
                    for bar in bars_zero:
                        if bar.contains(event)[0]:
                            bar.set_alpha(0.7)  # Highlight the bar
                        else:
                            bar.set_alpha(1.0)  # Reset other bars
                    canvas2.draw()

            fig2.canvas.mpl_connect("motion_notify_event", on_bar_hover)

            # Enable zoom and pan for bar chart
            from matplotlib.widgets import Cursor
            cursor = Cursor(ax2, useblit=True, color="red", linewidth=1)
            plt.tight_layout()

        # New Bar Chart for Average Values (specific columns only)
        if data is not None:
            # List of columns to include in the bar chart
            target_columns_avg = [
                "end_tidal_co2", "feed_vol", "oxygen_flow_rate", "fio2",
                "peep", "tidal_vol", "pip", "sip", "insp_time"
            ]

            # Filter the dataset to include only the target columns
            filtered_data_avg = data[target_columns_avg]

            # Calculate average values for each target column
            avg_values = filtered_data_avg.mean()
            columns_avg = avg_values.index.tolist()
            values_avg = avg_values.values

            # Bar Chart for Average Values
            fig3, ax3 = plt.subplots(figsize=(8, 5), dpi=80)  # Taller chart
            bars_avg = ax3.bar(columns_avg, values_avg, color="#FFC107")
            ax3.set_title("Average Values in Key Columns", fontsize=12, color=TEXT_COLOR)
            ax3.set_ylabel("Average Value", fontsize=10, color=TEXT_COLOR)
            ax3.set_xlabel("Columns", fontsize=10, color=TEXT_COLOR)
            ax3.tick_params(axis="x", rotation=90, colors=TEXT_COLOR)  # Rotate x-axis labels
            ax3.tick_params(axis="y", colors=TEXT_COLOR)
            ax3.set_facecolor(NAV_BAR_COLOR)
            fig3.patch.set_facecolor(NAV_BAR_COLOR)

            # Add interactivity to bar chart (tooltips and zoom/pan)
            for bar in bars_avg:
                height = bar.get_height()
                ax3.text(
                    bar.get_x() + bar.get_width() / 2, height,
                    f"{height:.2f}", ha="center", va="bottom", color=TEXT_COLOR
                )
                bar.set_picker(True)  # Enable picking for hover events

            canvas3 = FigureCanvasTkAgg(fig3, self.bar_frame_avg)
            canvas3.draw()
            canvas3.get_tk_widget().pack(expand=True, fill="both", padx=10, pady=10)

            # Connect hover event for bar chart
            def on_bar_hover(event):
                if event.inaxes == ax3:
                    for bar in bars_avg:
                        if bar.contains(event)[0]:
                            bar.set_alpha(0.7)  # Highlight the bar
                        else:
                            bar.set_alpha(1.0)  # Reset other bars
                    canvas3.draw()

            fig3.canvas.mpl_connect("motion_notify_event", on_bar_hover)

            # Enable zoom and pan for bar chart
            from matplotlib.widgets import Cursor
            cursor = Cursor(ax3, useblit=True, color="red", linewidth=1)
            plt.tight_layout()









class ViewPage(ctk.CTkFrame):
    def __init__(self, parent, controller):
        super().__init__(parent)
        self.controller = controller
        self.full_data = None
        self.current_filter = None
        self.configure(fg_color="transparent")

        # Search Entry
        self.search_var = tk.StringVar()
        self.search_var.trace_add("write", self._apply_filters)
        self.search_entry = ctk.CTkEntry(
            self, textvariable=self.search_var, placeholder_text="Search...", width=300
        )
        self.search_entry.pack(pady=5)

        # Filter Buttons Frame
        self.filter_frame = ctk.CTkFrame(self, fg_color="transparent")
        self.filter_frame.pack(pady=5)

        # Filter Buttons
        self.btn_referred = ctk.CTkButton(
            self.filter_frame, text="Referred", width=120,
            fg_color=("#DB3E39", "#821D1A"), hover_color="black",
            command=lambda: self._toggle_filter("Referred")
        )
        self.btn_referred.pack(side="left", padx=5)

        self.btn_need = ctk.CTkButton(
            self.filter_frame, text="Need Referred", width=120,
            fg_color=("#DB3E39", "#821D1A"), hover_color="black",
            command=lambda: self._toggle_filter("Need Referred")
        )
        self.btn_need.pack(side="left", padx=5)

        self.btn_not = ctk.CTkButton(
            self.filter_frame, text="Do Not Need Referred", width=160,
            fg_color=("#DB3E39", "#821D1A"), hover_color="black",
            command=lambda: self._toggle_filter("Do Not Need Referred")
        )
        self.btn_not.pack(side="left", padx=5)

        self.btn_reset = ctk.CTkButton(
            self.filter_frame, text="Reset Filters", width=120,
            fg_color=("#DB3E39", "#821D1A"), hover_color="black",
            command=self._reset_filters
        )
        self.btn_reset.pack(side="left", padx=5)

        # No data label
        self.no_data_label = ctk.CTkLabel(
            self,
            text="No data available. Please upload a CSV file.",
            font=("Arial", 16),
            text_color='black'
        )
        self.no_data_label.pack(pady=50)
        
        # Table Frame (initially hidden)
        self.table_frame = ctk.CTkFrame(self, fg_color="white")
        self.table_frame.pack_forget()
        self.tree = None

    def update_table(self, data):
        """Update the table with new data or show empty state"""
        self.full_data = data.copy() if data is not None else None
        self.current_filter = None
        
        # Clear any existing table
        self._clear_table()
        
        # Show/hide elements based on data availability
        if data is None or data.empty:
            self.no_data_label.pack(pady=50)
            self.table_frame.pack_forget()
            self.search_entry.pack_forget()
            self.filter_frame.pack_forget()
        else:
            self.no_data_label.pack_forget()
            self.search_entry.pack(pady=5)
            self.filter_frame.pack(pady=5)
            self.table_frame.pack(fill="both", expand=True, padx=10, pady=10)
            self._populate_table(data)

    def _clear_table(self):
        """Clear the existing table widgets"""
        if hasattr(self, 'tree') and self.tree:
            self.tree.destroy()
            self.tree = None
        for widget in self.table_frame.winfo_children():
            widget.destroy()

    def _populate_table(self, data):
        """Create the table with data"""
         # Define the desired column order
        desired_order = [
            "encounterId","Result", "end_tidal_co2","feed_vol",	"feed_vol_adm",	"fio2","fio2_ratio","insp_time","oxygen_flow_rate",
                "peep",	"pip","resp_rate","sip",	"tidal_vol","tidal_vol_actual",	"tidal_vol_kg",	"tidal_vol_spon","bmi"

            # Add any other columns in the order you prefer
        ]

        # Reorder the dataframe to match the desired column order
        data = data[desired_order]

        container = tk.Frame(self.table_frame, bg="white")
        container.pack(fill="both", expand=True)

        # Define columns in the desired order
        self.tree = ttk.Treeview(container, columns=desired_order, show="headings")
        self.tree.grid(row=0, column=0, sticky="nsew")

        # Scrollbars
        vsb = ttk.Scrollbar(container, orient="vertical", command=self.tree.yview)
        vsb.grid(row=0, column=1, sticky="ns")
        self.tree.configure(yscrollcommand=vsb.set)

        hsb = ttk.Scrollbar(container, orient="horizontal", command=self.tree.xview)
        hsb.grid(row=1, column=0, sticky="ew")
        self.tree.configure(xscrollcommand=hsb.set)

        container.rowconfigure(0, weight=1)
        container.columnconfigure(0, weight=1)

        # Set headers
        for col in data.columns:
            self.tree.heading(col, text=col)
            self.tree.column(col, anchor="center")

        # Insert rows
        for _, row in data.iterrows():
            values = list(row)
            if row["Result"] == "Need Referred":
                self.tree.insert("", "end", values=values, tags=("need_referred",))
            else:
                self.tree.insert("", "end", values=values)

        self.tree.tag_configure("need_referred", background="#D3D3D3", foreground="black")

    def _toggle_filter(self, filter_value):
        """Toggle the applied filter"""
        if self.current_filter == filter_value:
            self.current_filter = None
        else:
            self.current_filter = filter_value
        self._apply_filters()

    def _apply_filters(self, *args):
        if self.full_data is None:
            return
            
        filtered = self.full_data.copy()
        search_text = self.search_var.get().lower().strip()
        
        if search_text:
            filtered = filtered[filtered.astype(str)
                              .apply(lambda row: search_text in row.to_string().lower(), axis=1)]
        
        if self.current_filter:
            filtered = filtered[filtered["Result"].astype(str).str.lower() == self.current_filter.lower()]
        
        self._clear_table()
        if not filtered.empty:
            self._populate_table(filtered)

    def _reset_filters(self):
        """Reset all filters"""
        self.search_var.set("")
        self.current_filter = None
        if self.full_data is not None:
            self._clear_table()
            self._populate_table(self.full_data)
# ------------------ Main Application Class ------------------
class App(ctk.CTk):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.title("Medical Dashboard")
        self.minsize(appwidth, appheight)
        center_window(self, appwidth, appheight)
        
        self.new_data = None
        self.model = self.load_model()
        
        # Navigation bar centered in one row.
        self.nav_frame = Dashboard_Frame(self, app=self)
        self.nav_frame.pack(pady=20)
        
        # Container for pages (Upload, Dashboard, and View).
        self.page_container = ctk.CTkFrame(self, fg_color="transparent")
        self.page_container.pack(expand=True, fill="both")
        
        self.pages = {}
        self.pages["upload"] = UploadPage(self.page_container, self)
        self.pages["dashboard"] = DashboardPage(self.page_container, self)
        self.pages["view"] = ViewPage(self.page_container, self)
        self.pages["help"] = HelpPage(self.page_container, self)
        
        # Initially display the Upload page.
        self.show_page("upload")
    
    def load_model(self): 
        try:
            model = joblib.load(model_path)
            return model
        except Exception as e:
            messagebox.showerror("Error", f"Could not load model: {e}")
            return None
    
    def upload_csv_file(self):
        file_path = filedialog.askopenfilename(
            title="Select a CSV File", filetypes=[("CSV Files", "*.csv")]
        )
        if file_path:
            try:
                self.new_data = pd.read_csv(file_path)
                if "encounterId" not in self.new_data.columns:
                    self.new_data["encounterId"] = range(1, len(self.new_data) + 1)
                
                if self.model:
                    y_pred_new = self.model.predict(self.new_data)
                    self.new_data["Prediction"] = y_pred_new
                
                
                if "referral" in self.new_data.columns:
                    def determine_result(row):
                        if row["Prediction"] == 1:
                            if row["referral"] == 1:
                                return "Referred"
                            else:
                                return "Need Referred"
                        else:
                            return "Do Not Need Referred"
                    self.new_data["Result"] = self.new_data.apply(determine_result, axis=1)
                # ---------------------------------------------------
                
                # Replace all null values with zero
                self.new_data.fillna(0, inplace=True)
                self.pages["upload"].set_export_state(ctk.NORMAL)
                self.pages["upload"].set_status(f"File uploaded: {file_path}")
                
                # Update Dashboard and View pages with the new data.
                self.pages["dashboard"].update_charts(self.new_data)
                self.pages["view"].update_table(self.new_data)
                # Automatically switch to the Dashboard page.
                self.show_page("dashboard")
            except Exception as e:
                print(f"Error loading CSV file: {e}")
                self.pages["upload"].set_status("Failed to load CSV file.")
    
    def export_csv_file(self):
        if self.new_data is not None:
            file_path = filedialog.asksaveasfilename(
                defaultextension=".csv", filetypes=[("CSV Files", "*.csv")]
            )
            if file_path:
                try:
                    self.new_data.to_csv(file_path, index=False)
                    messagebox.showinfo("Export Successful", f"File saved successfully at:\n{file_path}")
                except Exception as e:
                    messagebox.showerror("Export Failed", f"Error saving file: {e}")
        else:
            messagebox.showwarning("No Data", "No data to export. Please upload a CSV first.")
    
    def show_page(self, page_key):
        for key, page in self.pages.items():
            page.place_forget()
        self.pages[page_key].place(relx=0, rely=0, relwidth=1, relheight=1)


# ------------------ Help Page ------------------
class HelpPage(ctk.CTkFrame):
    def __init__(self, parent, controller):
        super().__init__(parent)
        self.controller = controller
        self.configure(fg_color="transparent")

        self.scrollable_frame = ctk.CTkScrollableFrame(self, fg_color="transparent")
        self.scrollable_frame.pack(expand=True, fill="both", padx=30, pady=30)

        self._add_heading("Help Guide – Medical Dashboard", size=24)
        self._add_paragraph(
            "This dashboard helps CCU staff identify patients who might require dietitian referrals. "
            "It uses amachine learning model to help in decision-making."
        )

        self._add_heading("Navigation Overview")
        self._add_paragraph(
            "- Dashboard: View overall statistics and graphs.\n"
            "- View: Browse patient data with filtering and search options.\n"
            "- Upload: Import new patient CSV data and generate report.\n"
            "- Help: Gives better information."
        )

        self._add_heading("Uploading Patient Data")
        self._add_paragraph(
            "1. Go to the Upload page.\n"
            "2. Click 'Upload CSV FILE' and select your data file best to use .csv.\n"
            "   - A 'referral' column (0 or 1) is recommended.\n"
            "3. After uploading, the system will:\n"
            "   - Predict referrals using the ML model.\n"
            "   - Add two columns:\n"
            "     • Prediction: Indicates referral need .\n"
            "     • Result:\n"
            "         - Referred: Already referred and predicted to need referral.\n"
            "         - Need Referred: Not referred but prediction indicates they should be.\n"
            "         - Do Not Need Referred: No referral predicted.\n"
            "4. After upload, you will be redirected to the Dashboard."
        )

        self._add_heading("Dashboard Page")
        self._add_paragraph(
            "Displays statistics and charts:\n"
            "- Number of Total patients\n"
            "- Referred patients\n"
            "- Patients needing referral\n"
            "- Patients not needing referral\n\n"
            "Includes interactive charts:\n"
            "- Pie Chart: Shows distribution of patient outcomes.\n"
            "- Bar Charts:\n"
            "   • Shows zero values in key columns .\n"
            "   • Average values of some data collumns."
        )

        self._add_heading("View Page")
        self._add_paragraph(
            "- Displays uploaded data in a searchable, scrollable table.\n"
            "- Features:\n"
            "   • Search bar for filtering.\n"
            "   • Filter buttons to isolate patient outcome categories.\n"
            "   • Highlighting for 'Need Referred' patients.\n"
            "   • Reset filters button to return to full dataset."
        )

        self._add_heading("Exporting Data")
        self._add_paragraph(
            "- Once data is uploaded and processed, use 'Download CSV FILE' on the Upload page to export new Database.\n"
            "Use 'Generate Report' button to dowloand the report containing the graphs and the information of 'Need Refferred' parients. "
            
        )



    def _add_heading(self, text, size=18):
        heading = ctk.CTkLabel(
            self.scrollable_frame,
            text=text,
            font=("Arial", size, "bold"),
            text_color='black',
            anchor="w",
            justify="left"
        )
        heading.pack(anchor="w", pady=(20 if size != 24 else 10, 5))

    def _add_paragraph(self, text):
        paragraph = ctk.CTkLabel(
            self.scrollable_frame,
            text=text,
            font=("Arial", 14),
            text_color='black',
            anchor="w",
            justify="left",
            wraplength=900
        )
        paragraph.pack(anchor="w", pady=(0, 10))





# ------------------ Example Transformer Class (if needed) ------------------
class DropColumns(BaseEstimator, TransformerMixin):
    def __init__(self, columns):
        self.columns = columns
    def fit(self, X, y=None):
        return self
    def transform(self, X):
        return X.drop(columns=self.columns, errors="ignore")

if __name__ == "__main__":
    app = App()
    app.mainloop()
